@@ -1,7 +1,8 @@
 #  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Copyright (c) 2008-2022
+#  National Technology and Engineering Solutions of Sandia, LLC
 #  Under the terms of Contract DE-NA0003525 with National Technology and
 #  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain
 #  rights in this software.
@@ -11,7 +12,7 @@
 from pyomo.core import (minimize, Constraint, TransformationFactory, value)
 from pyomo.core.base.constraint import ConstraintList
 from pyomo.opt import SolverFactory, SolutionStatus, SolverResults, SolverStatus
-from pyomo.contrib.gdpopt.util import SuppressInfeasibleWarning, copy_var_list_values, time_code
+from pyomo.contrib.gdpopt.util import SuppressInfeasibleWarning, copy_var_list_values, time_code, get_main_elapsed_time
 from pyomo.contrib.mindtpy.nlp_solve import solve_subproblem, handle_subproblem_optimal
 from pyomo.opt import TerminationCondition as tc
 from pyomo.contrib.mindtpy.util import generate_norm2sq_objective_function, set_solver_options
@@ -20,7 +21,22 @@ from pyomo.contrib.mindtpy.util import generate_norm1_norm_constraint
 
 
 def fp_converged(solve_data, config, discrete_only=True):
-    """Calculates the euclidean norm between the discretes in the mip and nlp models"""
+    """Calculates the euclidean norm between the discrete variables in the MIP and NLP models.
+
+    Parameters
+    ----------
+    solve_data : MindtPySolveData
+        Data container that holds solve-instance data.
+    config : ConfigBlock
+        The specific configurations for MindtPy.
+    discrete_only : bool, optional
+        Whether to only optimize on distance between the discrete variables, by default True.
+
+    Returns
+    -------
+    distance : float
+        The euclidean norm between the discrete variables in the MIP and NLP models.
+    """
     distance = (max((nlp_var.value - milp_var.value)**2
                     for (nlp_var, milp_var) in
                     zip(solve_data.working_model.MindtPy_utils.variable_list,
@@ -30,43 +46,39 @@ def fp_converged(solve_data, config, discrete_only=True):
 
 
 def solve_fp_subproblem(solve_data, config):
-    """
-    Solves the feasibility pump NLP
+    """Solves the feasibility pump NLP subproblem.
 
     This function sets up the 'fp_nlp' by relax integer variables.
     precomputes dual values, deactivates trivial constraints, and then solves NLP model.
 
     Parameters
     ----------
-    solve_data: MindtPy Data Container
-        data container that holds solve-instance data
-    config: ConfigBlock
-        contains the specific configurations for the algorithm
+    solve_data : MindtPySolveData
+        Data container that holds solve-instance data.
+    config : ConfigBlock
+        The specific configurations for MindtPy.
 
     Returns
     -------
-    fp_nlp: Pyomo model
-        Fixed-NLP from the model
-    results: Pyomo results object
-        result from solving the Fixed-NLP
+    fp_nlp : Pyomo model
+        Fixed-NLP from the model.
+    results : SolverResults
+        Results from solving the fixed-NLP subproblem.
     """
-
     fp_nlp = solve_data.working_model.clone()
     MindtPy = fp_nlp.MindtPy_utils
-    config.logger.info('FP-NLP %s: Solve feasibility pump NLP subproblem.'
-                       % (solve_data.fp_iter,))
 
     # Set up NLP
     fp_nlp.MindtPy_utils.objective_list[-1].deactivate()
     if solve_data.objective_sense == minimize:
         fp_nlp.improving_objective_cut = Constraint(
-            expr=fp_nlp.MindtPy_utils.objective_value <= solve_data.UB)
+            expr=sum(fp_nlp.MindtPy_utils.objective_value[:]) <= solve_data.primal_bound)
     else:
         fp_nlp.improving_objective_cut = Constraint(
-            expr=fp_nlp.MindtPy_utils.objective_value >= solve_data.LB)
+            expr=sum(fp_nlp.MindtPy_utils.objective_value[:]) >= solve_data.primal_bound)
 
     # Add norm_constraint, which guarantees the monotonicity of the norm objective value sequence of all iterations
-    # Ref: Paper 'A storm of feasibility pumps for nonconvex MINLP'
+    # Ref: Paper 'A storm of feasibility pumps for nonconvex MINLP'   https://doi.org/10.1007/s10107-012-0608-x
     # the norm type is consistant with the norm obj of the FP-main problem.
     if config.fp_norm_constraint:
         generate_norm_constraint(fp_nlp, solve_data, config)
@@ -97,9 +109,17 @@ def solve_fp_subproblem(solve_data, config):
 
 
 def handle_fp_subproblem_optimal(fp_nlp, solve_data, config):
-    """Copies result to working model, updates bound, adds OA cut, no_good cut
-    and increasing objective cut and stores best solution if new one is best
-    Also calculates the duals
+    """Copies the solution to the working model, updates bound, adds OA cuts / no-good cuts /
+    increasing objective cut, calculates the duals and stores incumbent solution if it has been improved.
+
+    Parameters
+    ----------
+    fp_nlp : Pyomo model
+        The feasibility pump NLP subproblem.
+    solve_data : MindtPySolveData
+        Data container that holds solve-instance data.
+    config : ConfigBlock
+        The specific configurations for MindtPy.
     """
     copy_var_list_values(
         fp_nlp.MindtPy_utils.variable_list,
@@ -125,30 +145,30 @@ def handle_fp_subproblem_optimal(fp_nlp, solve_data, config):
 
 
 def fp_loop(solve_data, config):
-    """
-    Feasibility pump loop 
+    """Feasibility pump loop.
 
-    This is the outermost function for the algorithms in this package; this function controls the progression of
-    solving the model.
+    This is the outermost function for the algorithms in this package; this function
+    controls the progression of solving the model.
 
     Parameters
     ----------
-    solve_data: MindtPy Data Container
-        data container that holds solve-instance data
-    config: ConfigBlock
-        contains the specific configurations for the algorithm
+    solve_data : MindtPySolveData
+        Data container that holds solve-instance data.
+    config : ConfigBlock
+        The specific configurations for MindtPy.
+
+    Raises
+    ------
+    ValueError
+        MindtPy unable to handle the termination condition of the FP-NLP subproblem.
     """
     while solve_data.fp_iter < config.fp_iteration_limit:
 
-        config.logger.info(
-            '---Feasibility Pump Iteration %s---'
-            % solve_data.fp_iter)
-
         solve_data.mip_subiter = 0
         # solve MILP main problem
         feas_main, feas_main_results = solve_main(
             solve_data, config, fp=True)
-        fp_should_terminate = handle_feas_main_tc(
+        fp_should_terminate = handle_fp_main_tc(
             feas_main_results, solve_data, config)
         if fp_should_terminate:
             break
@@ -159,8 +179,11 @@ def fp_loop(solve_data, config):
             solve_data, config)
 
         if fp_nlp_result.solver.termination_condition in {tc.optimal, tc.locallyOptimal, tc.feasible}:
-            config.logger.info('FP-NLP %s: Distance-OBJ: %s'
-                               % (solve_data.fp_iter, value(fp_nlp.MindtPy_utils.fp_nlp_obj)))
+            config.logger.info(solve_data.log_formatter.format(
+                solve_data.fp_iter, 'FP-NLP', value(
+                    fp_nlp.MindtPy_utils.fp_nlp_obj),
+                solve_data.primal_bound, solve_data.dual_bound, solve_data.rel_gap,
+                get_main_elapsed_time(solve_data.timing)))
             handle_fp_subproblem_optimal(fp_nlp, solve_data, config)
         elif fp_nlp_result.solver.termination_condition in {tc.infeasible, tc.noSolution}:
             config.logger.error('Feasibility pump NLP subproblem infeasible')
@@ -202,17 +225,16 @@ def fp_loop(solve_data, config):
 
 
 def add_orthogonality_cuts(solve_data, config):
-    """
-    Add orthogonality cuts
+    """Add orthogonality cuts.
 
     This function adds orthogonality cuts to avoid cycling when the independence constraint qualification is not satisfied.
 
     Parameters
     ----------
-    solve_data: MindtPy Data Container
-        data container that holds solve-instance data
-    config: ConfigBlock
-        contains the specific configurations for the algorithm
+    solve_data : MindtPySolveData
+        Data container that holds solve-instance data.
+    config : ConfigBlock
+        The specific configurations for MindtPy.
     """
     mip_integer_vars = solve_data.mip.MindtPy_utils.discrete_variable_list
     nlp_integer_vars = solve_data.working_model.MindtPy_utils.discrete_variable_list
@@ -228,6 +250,17 @@ def add_orthogonality_cuts(solve_data, config):
 
 
 def generate_norm_constraint(fp_nlp, solve_data, config):
+    """Generate the norm constraint for the FP-NLP subproblem.
+
+    Parameters
+    ----------
+    fp_nlp : Pyomo model
+        The feasibility pump NLP subproblem.
+    solve_data : MindtPySolveData
+        Data container that holds solve-instance data.
+    config : ConfigBlock
+        The specific configurations for MindtPy.
+    """
     if config.fp_main_norm == 'L1':
         # TODO: check if we can access the block defined in FP-main problem
         generate_norm1_norm_constraint(
@@ -243,11 +276,28 @@ def generate_norm_constraint(fp_nlp, solve_data, config):
             fp_nlp.norm_constraint.add(nlp_var - mip_var.value <= rhs)
 
 
-def handle_feas_main_tc(feas_main_results, solve_data, config):
+def handle_fp_main_tc(feas_main_results, solve_data, config):
+    """Handle the termination condition of the feasibility pump main problem.
+
+    Parameters
+    ----------
+    feas_main_results : SolverResults
+        The results from solving the FP main problem.
+    solve_data : MindtPySolveData
+        Data container that holds solve-instance data.
+    config : ConfigBlock
+        The specific configurations for MindtPy.
+
+    Returns
+    -------
+    bool
+        True if FP loop should terminate, False otherwise.
+    """
     if feas_main_results.solver.termination_condition is tc.optimal:
-        config.logger.info(
-            'FP-MIP %s: Distance-OBJ: %s'
-            % (solve_data.fp_iter, value(solve_data.mip.MindtPy_utils.fp_mip_obj)))
+        config.logger.info(solve_data.log_formatter.format(
+            solve_data.fp_iter, 'FP-MIP', value(
+                solve_data.mip.MindtPy_utils.fp_mip_obj),
+            solve_data.primal_bound, solve_data.dual_bound, solve_data.rel_gap, get_main_elapsed_time(solve_data.timing)))
         return False
     elif feas_main_results.solver.termination_condition is tc.maxTimeLimit:
         config.logger.warning('FP-MIP reaches max TimeLimit')
