@@ -1,7 +1,8 @@
 #  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Copyright (c) 2008-2022
+#  National Technology and Engineering Solutions of Sandia, LLC
 #  Under the terms of Contract DE-NA0003525 with National Technology and
 #  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain
 #  rights in this software.
@@ -16,7 +17,7 @@ import os
 
 logger = logging.getLogger(__name__)
 
-CURRENT_INTERFACE_VERSION = 2
+CURRENT_INTERFACE_VERSION = 3
 
 class _NotSet:
     pass
@@ -38,6 +39,11 @@ def _LoadASLInterface(libname):
     except AttributeError:
         interface_version = 1
 
+    # ASL version
+    if interface_version >= 3:
+        ASLib.EXTERNAL_get_asl_date.argtypes = []
+        ASLib.EXTERNAL_get_asl_date.restype = ctypes.c_long
+
     # constructor
     ASLib.EXTERNAL_AmplInterface_new.argtypes = [ctypes.c_char_p]
     ASLib.EXTERNAL_AmplInterface_new.restype = ctypes.c_void_p
@@ -201,6 +207,7 @@ class AmplInterface(object):
     libname = _NotSet
     ASLib = None
     interface_version = None
+    asl_date = None
 
     @classmethod
     def available(cls):
@@ -228,6 +235,11 @@ class AmplInterface(object):
         if AmplInterface.ASLib is None:
             AmplInterface.ASLib, AmplInterface.interface_version \
                 = _LoadASLInterface(AmplInterface.libname)
+            if AmplInterface.interface_version >= 3:
+                AmplInterface.asl_date \
+                    = AmplInterface.ASLib.EXTERNAL_get_asl_date()
+            else:
+                AmplInterface.asl_date = 0
 
         if filename is not None:
             if nl_buffer is not None:
