@@ -1,7 +1,8 @@
 # ______________________________________________________________________________
 #
 # Pyomo: Python Optimization Modeling Objects
-# Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+# Copyright (c) 2008-2022
+#  National Technology and Engineering Solutions of Sandia, LLC
 # Under the terms of Contract DE-NA0003525 with National Technology and
 # Engineering Solutions of Sandia, LLC, the U.S. Government retains certain
 # rights in this software.
@@ -600,7 +601,7 @@ class SensitivityInterface(object):
                                                 active=True,
                                                 descend_into=True)):
             tempName = unique_component_name(block, obj.local_name)
-            new_expr = param_replacer.dfs_postorder_stack(obj.expr)
+            new_expr = param_replacer.walk_expression(obj.expr)
             block.add_component(tempName, Objective(expr=new_expr))
             new_old_comp_map[block.component(tempName)] = obj
             obj.deactivate()
@@ -615,16 +616,16 @@ class SensitivityInterface(object):
         last_idx = 0
         for con in old_con_list:
             if (con.equality or con.lower is None or con.upper is None):
-                new_expr = param_replacer.dfs_postorder_stack(con.expr)
+                new_expr = param_replacer.walk_expression(con.expr)
                 block.constList.add(expr=new_expr)
                 last_idx += 1
                 new_old_comp_map[block.constList[last_idx]] = con
             else:
                 # Constraint must be a ranged inequality, break into
                 # separate constraints
-                new_body = param_replacer.dfs_postorder_stack(con.body)
-                new_lower = param_replacer.dfs_postorder_stack(con.lower)
-                new_upper = param_replacer.dfs_postorder_stack(con.upper)
+                new_body = param_replacer.walk_expression(con.body)
+                new_lower = param_replacer.walk_expression(con.lower)
+                new_upper = param_replacer.walk_expression(con.upper)
 
                 # Add constraint for lower bound
                 block.constList.add(expr=(new_lower <= new_body))
