@@ -1,7 +1,8 @@
 #  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Copyright (c) 2008-2022
+#  National Technology and Engineering Solutions of Sandia, LLC
 #  Under the terms of Contract DE-NA0003525 with National Technology and
 #  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain
 #  rights in this software.
@@ -15,8 +16,8 @@ from pyomo.opt import SolverFactory, TerminationCondition
 from pyomo.solvers.plugins.solvers.GAMS import (
     GAMSShell, GAMSDirect, gdxcc_available
 )
-import pyutilib.th as unittest
-from pyutilib.misc import capture_output
+import pyomo.common.unittest as unittest
+from pyomo.common.tee import capture_output
 import os, shutil
 from tempfile import mkdtemp
 
